package com.engine.organization.thread;

import com.engine.organization.entity.department.po.DepartmentPO;
import com.engine.organization.entity.hrmresource.po.HrmResourcePO;
import com.engine.organization.entity.logview.bo.FieldBaseEquator;
import com.engine.organization.entity.map.JclOrgMap;
import com.engine.organization.enums.ModuleTypeEnum;
import com.engine.organization.mapper.department.DepartmentMapper;
import com.engine.organization.mapper.jclorgmap.JclOrgMapper;
import com.engine.organization.mapper.trigger.CompTriggerMapper;
import com.engine.organization.mapper.trigger.DepartmentTriggerMapper;
import com.engine.organization.util.OrganizationDateUtil;
import com.engine.organization.util.db.MapperProxyFactory;
import org.apache.commons.collections.CollectionUtils;
import weaver.common.DateUtil;
import weaver.hrm.passwordprotection.domain.HrmResource;

import java.sql.Date;
import java.util.Calendar;
import java.util.List;

/**
 * @author:dxfeng
 * @createTime: 2022/08/30
 * @version: 1.0
 */
public class DepartmentTriggerRunnable implements Runnable {

    private DepartmentPO oldDepartment;
    private final DepartmentPO newDepartment;
    private String oldFparentId;
    private String moveTarget;

    private CompTriggerMapper getCompTriggerMapper() {
        return MapperProxyFactory.getProxy(CompTriggerMapper.class);
    }

    private DepartmentTriggerMapper getDepartmentTriggerMapper() {
        return MapperProxyFactory.getProxy(DepartmentTriggerMapper.class);
    }

    public DepartmentTriggerRunnable(DepartmentPO oldDepartment, DepartmentPO newDepartment) {
        this.oldDepartment = oldDepartment;
        this.newDepartment = newDepartment;
    }

    public DepartmentTriggerRunnable(Long departmentId) {
        this.oldDepartment = new DepartmentPO();
        this.newDepartment = MapperProxyFactory.getProxy(DepartmentMapper.class).getDeptById(departmentId);
    }
    public DepartmentTriggerRunnable(String moveTarget,DepartmentPO newDepartment) {
        this.oldDepartment = new DepartmentPO();
        this.newDepartment = newDepartment;
        this.moveTarget = moveTarget;
    }

    public DepartmentTriggerRunnable(Integer oldFparentId, Long departmentId) {
        this.oldFparentId = oldFparentId.toString();
        this.newDepartment = MapperProxyFactory.getProxy(DepartmentMapper.class).getDeptById(departmentId);
    }

    public DepartmentTriggerRunnable(DepartmentPO newDepartment) {
        this.oldDepartment = new DepartmentPO();
        this.newDepartment = newDepartment;
        this.newDepartment.setDeleteType(1);
    }

    @Override
    public void run() {
        FieldBaseEquator fieldBaseEquator = new FieldBaseEquator();
        List<String> diffFields = fieldBaseEquator.getDiffFieldList(oldDepartment, newDepartment);
        if (CollectionUtils.isEmpty(diffFields)) {
            return;
        }

        // 判断
        if (diffFields.contains("deptName") || diffFields.contains("deptPrincipal") || diffFields.contains("parentComp") || diffFields.contains("parentDept") || diffFields.contains("forbiddenTag") || diffFields.contains("deleteType")) {
            JclOrgMap jclMap = new JclOrgMap();
            jclMap.setFType(2);
            int st = 100000000;

            // 更新逻辑
            jclMap.setFObjId(newDepartment.getId().intValue());
            jclMap.setId(newDepartment.getId().intValue() + st);
            jclMap.setUuid(newDepartment.getUuid());
            jclMap.setFNumber(newDepartment.getDeptNo());
            jclMap.setFName(newDepartment.getDeptName());
            jclMap.setFLeader(null == newDepartment.getDeptPrincipal() ? null : newDepartment.getDeptPrincipal().intValue());
            jclMap.setFParentId(null == newDepartment.getParentDept() ? newDepartment.getParentComp().intValue() : newDepartment.getParentDept().intValue() + st);
            jclMap.setFObjParentId(null == newDepartment.getParentDept() ? newDepartment.getParentComp().intValue() : newDepartment.getParentDept().intValue());

            jclMap.setFEcId(getDepartmentTriggerMapper().getEcDepartmentIdByUuid(jclMap.getUuid()));

            jclMap.setFClass(0);
            jclMap.setFClassName("行政维度");
            HrmResource hrmResourceById = getCompTriggerMapper().getHrmResourceById(jclMap.getFLeader());

            if (null != hrmResourceById) {
                jclMap.setFLeaderImg(hrmResourceById.getMessagerurl());
                jclMap.setFLeaderName(hrmResourceById.getLastname());
                jclMap.setFLeaderJobId(hrmResourceById.getJobtitle());
            }

            jclMap.setFLeaderJob(getCompTriggerMapper().getJobTitleMarkById(jclMap.getFLeaderJobId()));


            HrmResourcePO resourceByEcId = getCompTriggerMapper().getResourceByEcId(jclMap.getFLeader());
            if (null != resourceByEcId) {
                jclMap.setFLeaderSt(resourceByEcId.getJobGrade());
                jclMap.setFLeaderLv(resourceByEcId.getJobLevel());
            }
            String currentDate = OrganizationDateUtil.getFormatLocalDate(new java.util.Date());
            jclMap.setFDateBegin(new Date(OrganizationDateUtil.stringToDate(currentDate).getTime()));
            jclMap.setFDateEnd(new Date(OrganizationDateUtil.stringToDate("2099-12-31").getTime()));

            // 获取当前生效的本部门map记录
            JclOrgMap jclOrgMapByObjID = MapperProxyFactory.getProxy(JclOrgMapper.class).getJclOrgMapByObjID(currentDate, ModuleTypeEnum.departmentfielddefined.getValue().toString(), jclMap.getFObjId().toString());

            // 取出以该部门为上级部门的在编、在岗数,转移无需计算
            JclOrgMap jclOrgMap = MapperProxyFactory.getProxy(JclOrgMapper.class).getSumPlanAndJobByFParentId(currentDate, jclMap.getId().toString());
            if (null != moveTarget) {
                jclOrgMap = null;
            }
            if (null != jclOrgMapByObjID) {
                if (null != jclOrgMap) {
                    jclMap.setFPlan(jclOrgMapByObjID.getFPlan() + jclOrgMap.getFPlan());
                    jclMap.setFOnJob(jclOrgMapByObjID.getFOnJob() + jclOrgMap.getFOnJob());
                } else {
                    jclMap.setFPlan(jclOrgMapByObjID.getFPlan());
                    jclMap.setFOnJob(jclOrgMapByObjID.getFOnJob());
                }
            } else {
                jclMap.setFPlan(0);
                jclMap.setFOnJob(0);
            }
            jclMap.setFIsVitual(0);

            Calendar cal = Calendar.getInstance();
            cal.setTime(jclMap.getFDateBegin());
            Calendar calendar = DateUtil.addDay(cal, -1);
            Date time = new Date(calendar.getTime().getTime());
            getCompTriggerMapper().deleteMap(jclMap.getFType(), jclMap.getFObjId(), jclMap.getFDateBegin());
            getCompTriggerMapper().updateMap(jclMap.getFType(), jclMap.getFObjId(), jclMap.getFDateBegin(), time);

            if (1 != newDepartment.getDeleteType() && 1 != newDepartment.getForbiddenTag()) {
                MapperProxyFactory.getProxy(JclOrgMapper.class).insertMap(jclMap);
                if(null != jclOrgMapByObjID) {
                    updateParentPlanAndJob(currentDate, jclOrgMapByObjID.getFParentId().toString());
                }
                if(null != moveTarget){
                    updateParentPlanAndJob(currentDate, moveTarget);
                }
                if (null != oldFparentId) {
                    updateParentPlanAndJob(currentDate, oldFparentId);
                }
            }
        }
    }

    private void updateParentPlanAndJob(String currentDate, String parentId) {
        // 获取上级部门
        JclOrgMap parentJclOrgMap = MapperProxyFactory.getProxy(JclOrgMapper.class).getJclOrgMapById(currentDate, parentId);
        if (null != parentJclOrgMap) {
            // 上级部门当前在编、在岗数
            JclOrgMap jclOrgMapSum = MapperProxyFactory.getProxy(JclOrgMapper.class).getSumPlanAndJobByFParentId(currentDate, parentJclOrgMap.getId().toString());

            // 编制表jcl_org_staff中的编制数+下级编制数，工具类判断
            if(null != jclOrgMapSum){
                parentJclOrgMap.setFPlan(jclOrgMapSum.getFPlan());
                parentJclOrgMap.setFOnJob(jclOrgMapSum.getFOnJob());
            }else{
                parentJclOrgMap.setFPlan(0);
                parentJclOrgMap.setFOnJob(0);
            }
            parentJclOrgMap.setFDateBegin(new Date(OrganizationDateUtil.stringToDate(currentDate).getTime()));
            parentJclOrgMap.setFDateEnd(new Date(OrganizationDateUtil.stringToDate("2099-12-31").getTime()));

            Calendar cal = Calendar.getInstance();
            cal.setTime(parentJclOrgMap.getFDateBegin());
            Calendar calendar = DateUtil.addDay(cal, -1);
            Date time = new Date(calendar.getTime().getTime());
            getCompTriggerMapper().deleteMap(parentJclOrgMap.getFType(), parentJclOrgMap.getFObjId(), parentJclOrgMap.getFDateBegin());
            getCompTriggerMapper().updateMap(parentJclOrgMap.getFType(), parentJclOrgMap.getFObjId(), parentJclOrgMap.getFDateBegin(), time);
            MapperProxyFactory.getProxy(JclOrgMapper.class).insertMap(parentJclOrgMap);
            if (null != parentJclOrgMap.getFParentId() && -1 != parentJclOrgMap.getFParentId()) {
                updateParentPlanAndJob(currentDate, parentJclOrgMap.getFParentId().toString());
            }
        }
    }
}
